@@ -20,8 +20,6 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 	try {
 		handle_rpc_json(transport);
 
-		loading_set_progress(25);
-
 		console.log("headlines_callback2 [offset=" + offset + "] B:" + background + " I:" + infscroll_req);
 
 		var is_cat = false;
@@ -43,13 +41,12 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 			if (background) {
 				var content = reply['headlines']['content'];
 
-				if (getInitParam("cdm_auto_catchup") == 1) {
-					content = content + "<div id='headlines-spacer'></div>";
-				}
+				content = content + "<div id='headlines-spacer'></div>";
 				return;
 			}
 
-			setActiveFeedId(feed_id, is_cat);
+			if (feed_id != getActiveFeedId() || is_cat != activeFeedIsCat())
+				return;
 
 			/* dijit.getEnclosingWidget(
 				document.forms["main_toolbar_form"].update).attr('disabled',
@@ -58,9 +55,18 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 			try {
 				if (infscroll_req == false) {
 					$("headlines-frame").scrollTop = 0;
+
+					Element.hide("floatingTitle");
+					$("floatingTitle").setAttribute("rowid", 0);
+					$("floatingTitle").innerHTML = "";
 				}
 			} catch (e) { };
 
+			$("headlines-frame").removeClassName("cdm");
+			$("headlines-frame").removeClassName("normal");
+
+			$("headlines-frame").addClassName(isCdmMode() ? "cdm" : "normal");
+
 			var headlines_count = reply['headlines-info']['count'];
 
 			vgroup_last_feed = reply['headlines-info']['vgroup_last_feed'];
@@ -92,11 +98,9 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 					}
 				});
 
-				if (getInitParam("cdm_auto_catchup") == 1) {
-					var hsp = $("headlines-spacer");
-					if (!hsp) hsp = new Element("DIV", {"id": "headlines-spacer"});
-					dijit.byId('headlines-frame').domNode.appendChild(hsp);
-				}
+				var hsp = $("headlines-spacer");
+				if (!hsp) hsp = new Element("DIV", {"id": "headlines-spacer"});
+				dijit.byId('headlines-frame').domNode.appendChild(hsp);
 
 				initHeadlinesMenu();
 
@@ -139,8 +143,6 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 
 					if (!hsp) hsp = new Element("DIV", {"id": "headlines-spacer"});
 
-					fixHeadlinesOrder(getLoadedArticleIds());
-
 					if (getInitParam("cdm_auto_catchup") == 1) {
 						c.domNode.appendChild(hsp);
 					}
@@ -159,16 +161,10 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 					initHeadlinesMenu();
 
 					new_elems.each(function(child) {
-						var cb = dijit.byId(child.id.replace("RROW-", "RCHK-"));
+						dojo.parser.parse(child);
 
-						if (!cb) {
-							dojo.parser.parse(child);
-
-							if (!Element.visible(child))
-								new Effect.Appear(child, { duration : 0.5 });
-						} else {
-							c.domNode.removeChild(child);
-						}
+						if (!Element.visible(child))
+							new Effect.Appear(child, { duration : 0.5 });
 					});
 
 				} else {
@@ -209,6 +205,16 @@ function headlines_callback2(transport, offset, background, infscroll_req) {
 
 		unpackVisibleHeadlines();
 
+		// if we have some more space in the buffer, why not try to fill it
+
+		if (!_infscroll_disable && $("headlines-spacer") &&
+				$("headlines-spacer").offsetTop < $("headlines-frame").offsetHeight) {
+
+			window.setTimeout(function() {
+				loadMoreHeadlines();
+			}, 250);
+		}
+
 		notify("");
 
 	} catch (e) {
@@ -227,6 +233,8 @@ function render_article(article) {
 			c.domNode.scrollTop = 0;
 		} catch (e) { };
 
+		PluginHost.run(PluginHost.HOOK_ARTICLE_RENDERED, article);
+
 		c.attr('content', article);
 
 		correctHeadlinesOffset(getActiveArticleId());
@@ -240,7 +248,7 @@ function render_article(article) {
 	}
 }
 
-function showArticleInHeadlines(id) {
+function showArticleInHeadlines(id, noexpand) {
 
 	try {
 		selectArticles("none");
@@ -251,7 +259,8 @@ function showArticleInHeadlines(id) {
 
 		var article_is_unread = crow.hasClassName("Unread");
 
-		crow.removeClassName("Unread");
+		if (!noexpand)
+			crow.removeClassName("Unread");
 		crow.addClassName("active");
 
 		selectArticles('none');
@@ -267,7 +276,7 @@ function showArticleInHeadlines(id) {
 
 		markHeadline(id);
 
-		if (article_is_unread)
+		if (article_is_unread && !noexpand)
 			_force_scheduled_update = true;
 
 	} catch (e) {
@@ -335,7 +344,7 @@ function article_callback2(transport, id) {
 	}
 }
 
-function view(id) {
+function view(id, activefeed, noexpand) {
 	try {
 		var oldrow = $("RROW-" + getActiveArticleId());
 		if (oldrow) oldrow.removeClassName("active");
@@ -343,6 +352,11 @@ function view(id) {
 		var crow = $("RROW-" + id);
 
 		if (!crow) return;
+		if (noexpand) {
+			setActiveArticleId(id);
+			showArticleInHeadlines(id, noexpand);
+			return;
+		}
 
 		console.log("loading article: " + id);
 
@@ -431,21 +445,41 @@ function toggleMark(id, client_only) {
 	try {
 		var query = "?op=rpc&id=" + id + "&method=mark";
 
-		var img = $("FMPIC-" + id);
+		var row = $("RROW-" + id);
+		if (!row) return;
 
-		if (!img) return;
+		var imgs = [];
 
-		if (img.src.match("mark_unset")) {
-			img.src = img.src.replace("mark_unset", "mark_set");
-			img.alt = __("Unstar article");
-			query = query + "&mark=1";
+		var row_imgs = row.getElementsByClassName("markedPic");
 
-		} else {
-			img.src = img.src.replace("mark_set", "mark_unset");
-			img.alt = __("Star article");
-			query = query + "&mark=0";
+		for (var i = 0; i < row_imgs.length; i++)
+			imgs.push(row_imgs[i]);
+
+		var ft = $("floatingTitle");
+
+		if (ft && ft.getAttribute("rowid") == "RROW-" + id) {
+			var fte = ft.getElementsByClassName("markedPic");
+
+			for (var i = 0; i < fte.length; i++)
+				imgs.push(fte[i]);
 		}
 
+		for (i = 0; i < imgs.length; i++) {
+			var img = imgs[i];
+
+			if (!row.hasClassName("marked")) {
+				img.src = img.src.replace("mark_unset", "mark_set");
+				img.alt = __("Unstar article");
+				query = query + "&mark=1";
+			} else {
+				img.src = img.src.replace("mark_set", "mark_unset");
+				img.alt = __("Star article");
+				query = query + "&mark=0";
+			}
+		}
+
+		row.toggleClassName("marked");
+
 		if (!client_only) {
 			new Ajax.Request("backend.php", {
 				parameters: query,
@@ -469,22 +503,44 @@ function togglePub(id, client_only, no_effects, note) {
 			query = query + "&note=undefined";
 		}
 
-		var img = $("FPPIC-" + id);
+		var row = $("RROW-" + id);
+		if (!row) return;
 
-		if (!img) return;
+		var imgs = [];
 
-		if (img.src.match("pub_unset") || note != undefined) {
-			img.src = img.src.replace("pub_unset", "pub_set");
-			img.alt = __("Unpublish article");
-			query = query + "&pub=1";
+		var row_imgs = row.getElementsByClassName("pubPic");
 
-		} else {
-			img.src = img.src.replace("pub_set", "pub_unset");
-			img.alt = __("Publish article");
+		for (var i = 0; i < row_imgs.length; i++)
+			imgs.push(row_imgs[i]);
 
-			query = query + "&pub=0";
+		var ft = $("floatingTitle");
+
+		if (ft && ft.getAttribute("rowid") == "RROW-" + id) {
+			var fte = ft.getElementsByClassName("pubPic");
+
+			for (var i = 0; i < fte.length; i++)
+				imgs.push(fte[i]);
 		}
 
+		for (i = 0; i < imgs.length; i++) {
+			var img = imgs[i];
+
+			if (!row.hasClassName("published") || note != undefined) {
+				img.src = img.src.replace("pub_unset", "pub_set");
+				img.alt = __("Unpublish article");
+				query = query + "&pub=1";
+			} else {
+				img.src = img.src.replace("pub_set", "pub_unset");
+				img.alt = __("Publish article");
+				query = query + "&pub=0";
+			}
+		}
+
+		if (note != undefined)
+			row.addClassName("published");
+		else
+			row.toggleClassName("published");
+
 		if (!client_only) {
 			new Ajax.Request("backend.php", {
 				parameters: query,
@@ -498,7 +554,7 @@ function togglePub(id, client_only, no_effects, note) {
 	}
 }
 
-function moveToPost(mode, noscroll) {
+function moveToPost(mode, noscroll, noexpand) {
 
 	try {
 
@@ -545,13 +601,13 @@ function moveToPost(mode, noscroll) {
 						scrollArticle(ctr.offsetHeight/4);
 
 					} else if (next_id) {
-						cdmExpandArticle(next_id);
+						cdmExpandArticle(next_id, noexpand);
 						cdmScrollToArticleId(next_id, true);
 					}
 
 				} else if (next_id) {
 					correctHeadlinesOffset(next_id);
-					view(next_id, getActiveFeedId());
+					view(next_id, getActiveFeedId(), noexpand);
 				}
 			}
 		}
@@ -566,10 +622,10 @@ function moveToPost(mode, noscroll) {
 
 					if (!getInitParam("cdm_expanded")) {
 
-						if (!noscroll && article.offsetTop < ctr.scrollTop) {
+						if (!noscroll && article && article.offsetTop < ctr.scrollTop) {
 							scrollArticle(-ctr.offsetHeight/4);
 						} else {
-							cdmExpandArticle(prev_id);
+							cdmExpandArticle(prev_id, noexpand);
 							cdmScrollToArticleId(prev_id, true);
 						}
 					} else {
@@ -578,17 +634,17 @@ function moveToPost(mode, noscroll) {
 							scrollArticle(-ctr.offsetHeight/3);
 						} else if (!noscroll && prev_article &&
 								prev_article.offsetTop < ctr.scrollTop) {
-							cdmExpandArticle(prev_id);
+							cdmExpandArticle(prev_id, noexpand);
 							scrollArticle(-ctr.offsetHeight/4);
 						} else if (prev_id) {
-							cdmExpandArticle(prev_id);
+							cdmExpandArticle(prev_id, noexpand);
 							cdmScrollToArticleId(prev_id, noscroll);
 						}
 					}
 
 				} else if (prev_id) {
 					correctHeadlinesOffset(prev_id);
-					view(prev_id, getActiveFeedId());
+					view(prev_id, getActiveFeedId(), noexpand);
 				}
 			}
 		}
@@ -600,11 +656,12 @@ function moveToPost(mode, noscroll) {
 
 function toggleSelected(id, force_on) {
 	try {
-
-		var cb = dijit.byId("RCHK-" + id);
 		var row = $("RROW-" + id);
 
 		if (row) {
+			var cb = dijit.getEnclosingWidget(
+					row.getElementsByClassName("rchk")[0]);
+
 			if (row.hasClassName('Selected') && !force_on) {
 				row.removeClassName('Selected');
 				if (cb) cb.attr("checked", false);
@@ -613,11 +670,33 @@ function toggleSelected(id, force_on) {
 				if (cb) cb.attr("checked", true);
 			}
 		}
+
+		updateSelectedPrompt();
 	} catch (e) {
 		exception_error("toggleSelected", e);
 	}
 }
 
+function updateSelectedPrompt() {
+	try {
+		var count = getSelectedArticleIds2().size();
+		var elem = $("selected_prompt");
+
+		if (elem) {
+			elem.innerHTML = ngettext("%d article selected",
+					"%d articles selected", count).replace("%d", count);
+
+			if (count > 0)
+				Element.show(elem);
+			else
+				Element.hide(elem);
+		}
+
+	} catch (e) {
+		exception_error("updateSelectedPrompt", e);
+	}
+}
+
 function toggleUnread_afh(effect) {
 	try {
 
@@ -887,7 +966,9 @@ function selectArticles(mode) {
 
 		children.each(function(child) {
 			var id = child.id.replace("RROW-", "");
-			var cb = dijit.byId("RCHK-" + id);
+
+			var cb = dijit.getEnclosingWidget(
+					child.getElementsByClassName("rchk")[0]);
 
 			if (mode == "all") {
 				child.addClassName("Selected");
@@ -901,9 +982,7 @@ function selectArticles(mode) {
 					if (cb) cb.attr("checked", false);
 				}
 			} else if (mode == "marked") {
-				var img = $("FMPIC-" + child.id.replace("RROW-", ""));
-
-				if (img && img.src.match("mark_set")) {
+				if (child.hasClassName("marked")) {
 					child.addClassName("Selected");
 					if (cb) cb.attr("checked", true);
 				} else {
@@ -911,9 +990,7 @@ function selectArticles(mode) {
 					if (cb) cb.attr("checked", false);
 				}
 			} else if (mode == "published") {
-				var img = $("FPPIC-" + child.id.replace("RROW-", ""));
-
-				if (img && img.src.match("pub_set")) {
+				if (child.hasClassName("published")) {
 					child.addClassName("Selected");
 					if (cb) cb.attr("checked", true);
 				} else {
@@ -936,6 +1013,8 @@ function selectArticles(mode) {
 			}
 		});
 
+		updateSelectedPrompt();
+
 	} catch (e) {
 		exception_error("selectArticles", e);
 	}
@@ -1145,6 +1224,7 @@ function cdmScrollToArticleId(id, force) {
 
 function setActiveArticleId(id) {
 	_active_article_id = id;
+	PluginHost.run(PluginHost.HOOK_ARTICLE_SET_ACTIVE, _active_article_id);
 }
 
 function getActiveArticleId() {
@@ -1171,6 +1251,8 @@ function unpackVisibleHeadlines() {
 					var cencw = $("CENCW-" + child.id.replace("RROW-", ""));
 
 					if (cencw) {
+						PluginHost.run(PluginHost.HOOK_ARTICLE_RENDERED_CDM, child);
+
 						cencw.innerHTML = htmlspecialchars_decode(cencw.innerHTML);
 						cencw.setAttribute('id', '');
 						Element.show(cencw);
@@ -1190,6 +1272,30 @@ function headlines_scroll_handler(e) {
 
 		unpackVisibleHeadlines();
 
+		// set topmost child in the buffer as active
+		if (getInitParam("cdm_auto_catchup") == 1 &&
+				(!isCdmMode() || getInitParam("cdm_expanded"))) {
+			var rows = $$("#headlines-frame > div[id*=RROW]");
+
+			for (var i = 0; i < rows.length; i++) {
+				var child = rows[i];
+
+				if ($("headlines-frame").scrollTop < child.offsetTop &&
+					child.offsetTop - $("headlines-frame").scrollTop < 100) {
+
+					if (_active_article_id) {
+						var row = $("RROW-" + _active_article_id);
+						if (row) row.removeClassName("active");
+					}
+
+					_active_article_id = child.id.replace("RROW-", "");
+					showArticleInHeadlines(_active_article_id, true);
+					updateSelectedPrompt();
+					break;
+				}
+			}
+		}
+
 		if (!_infscroll_disable) {
 			if ((hsp && e.scrollTop + e.offsetHeight >= hsp.offsetTop - hsp.offsetHeight) ||
 					(e.scrollHeight != 0 &&
@@ -1207,6 +1313,10 @@ function headlines_scroll_handler(e) {
 			if (hsp) hsp.innerHTML = "";
 		}
 
+		if (getInitParam("cdm_expanded") && isCdmMode()) {
+			updateFloatingTitle();
+		}
+
 		if (getInitParam("cdm_auto_catchup") == 1) {
 
 			// let's get DOM some time to settle down
@@ -1225,6 +1335,7 @@ function headlines_scroll_handler(e) {
 
 						//console.log("auto_catchup_batch: " + catchup_id_batch.toString());
 					}
+
 				});
 
 			if (catchup_id_batch.length > 0) {
@@ -1346,8 +1457,10 @@ function catchupRelativeToArticle(below, id) {
 	}
 }
 
-function cdmCollapseArticle(event, id) {
+function cdmCollapseArticle(event, id, unmark) {
 	try {
+		if (unmark == undefined) unmark = true;
+
 		var row = $("RROW-" + id);
 		var elem = $("CICD-" + id);
 
@@ -1358,15 +1471,22 @@ function cdmCollapseArticle(event, id) {
 		  	Element.hide(elem);
 			Element.show("CEXC-" + id);
 			Element.hide(collapse);
-			row.removeClassName("active");
 
-			markHeadline(id, false);
+			if (unmark) {
+				row.removeClassName("active");
 
-			if (id == getActiveArticleId()) {
-				setActiveArticleId(0);
+				markHeadline(id, false);
+
+				if (id == getActiveArticleId()) {
+					setActiveArticleId(0);
+				}
+
+				updateSelectedPrompt();
 			}
 
 			if (event) Event.stop(event);
+
+			PluginHost.run(PluginHost.HOOK_ARTICLE_COLLAPSED, id);
 		}
 
 	} catch (e) {
@@ -1374,7 +1494,7 @@ function cdmCollapseArticle(event, id) {
 	}
 }
 
-function cdmExpandArticle(id) {
+function cdmExpandArticle(id, noexpand) {
 	try {
 		console.log("cdmExpandArticle " + id);
 
@@ -1411,7 +1531,7 @@ function cdmExpandArticle(id) {
 
 		var cencw = $("CENCW-" + id);
 
-		if (!Element.visible(elem)) {
+		if (!Element.visible(elem) && !noexpand) {
 			if (cencw) {
 				cencw.innerHTML = htmlspecialchars_decode(cencw.innerHTML);
 				cencw.setAttribute('id', '');
@@ -1428,10 +1548,13 @@ function cdmExpandArticle(id) {
 		if (old_offset > new_offset)
 			$("headlines-frame").scrollTop -= (old_offset-new_offset);
 
-		toggleUnread(id, 0, true);
+		if (!noexpand)
+			toggleUnread(id, 0, true);
 		toggleSelected(id);
 		$("RROW-" + id).addClassName("active");
 
+		PluginHost.run(PluginHost.HOOK_ARTICLE_EXPANDED, id);
+
 	} catch (e) {
 		exception_error("cdmExpandArticle", e);
 	}
@@ -1439,26 +1562,6 @@ function cdmExpandArticle(id) {
 	return false;
 }
 
-function fixHeadlinesOrder(ids) {
-	try {
-		for (var i = 0; i < ids.length; i++) {
-			var e = $("RROW-" + ids[i]);
-
-			if (e) {
-				if (i % 2 == 0) {
-					e.removeClassName("even");
-					e.addClassName("odd");
-				} else {
-					e.removeClassName("odd");
-					e.addClassName("even");
-				}
-			}
-		}
-	} catch (e) {
-		exception_error("fixHeadlinesOrder", e);
-	}
-}
-
 function getArticleUnderPointer() {
 	return post_under_pointer;
 }
@@ -1539,7 +1642,6 @@ function dismissSelectedArticles() {
 		if (sel.length > 0)
 			selectionToggleUnread(false);
 
-		fixHeadlinesOrder(tmp);
 
 	} catch (e) {
 		exception_error("dismissSelectedArticles", e);
@@ -1564,8 +1666,6 @@ function dismissReadArticles() {
 			}
 		}
 
-		fixHeadlinesOrder(tmp);
-
 	} catch (e) {
 		exception_error("dismissSelectedArticles", e);
 	}
@@ -1700,7 +1800,8 @@ function markHeadline(id, marked) {
 
 	var row = $("RROW-" + id);
 	if (row) {
-		var check = dijit.byId("RCHK-" + id);
+		var check = dijit.getEnclosingWidget(
+				row.getElementsByClassName("rchk")[0]);
 
 		if (check) {
 			check.attr("checked", marked);
@@ -1857,7 +1958,7 @@ function initHeadlinesMenu() {
 				}}));
 
 		menu.addChild(new dijit.MenuItem({
-			label: __("Toggle marked"),
+			label: __("Toggle starred"),
 			onClick: function(event) {
 				var ids = getSelectedArticleIds2();
 				// cast to string
@@ -1956,34 +2057,6 @@ function initHeadlinesMenu() {
 	}
 }
 
-
-function player(elem) {
-	var aid = elem.getAttribute("audio-id");
-	var status = elem.getAttribute("status");
-
-	var audio = $(aid);
-
-	if (audio) {
-		if (status == 0) {
-			audio.play();
-			status = 1;
-			elem.innerHTML = __("Playing...");
-			elem.title = __("Click to pause");
-			elem.addClassName("playing");
-		} else {
-			audio.pause();
-			status = 0;
-			elem.innerHTML = __("Play");
-			elem.title = __("Click to play");
-			elem.removeClassName("playing");
-		}
-
-		elem.setAttribute("status", status);
-	} else {
-		alert("Your browser doesn't seem to support HTML5 audio.");
-	}
-}
-
 function cache_set(id, obj) {
 	//console.log("cache_set: " + id);
 	if (has_storage)
@@ -2121,3 +2194,35 @@ function openSelectedAttachment(elem) {
 		exception_error("openSelectedAttachment", e);
 	}
 }
+
+function updateFloatingTitle() {
+	try {
+		var hf = $("headlines-frame");
+
+		var elems = $$("#headlines-frame > div[id*=RROW]");
+
+		for (var i = 0; i < elems.length; i++) {
+			var child = elems[i];
+
+			if (child.offsetTop + child.offsetHeight > hf.scrollTop) {
+
+				var header = child.getElementsByClassName("cdmHeader")[0];
+
+				if (child.id != $("floatingTitle").getAttribute("rowid")) {
+					$("floatingTitle").setAttribute("rowid", child.id);
+					$("floatingTitle").innerHTML = header.innerHTML;
+				}
+
+				if (child.offsetTop < hf.scrollTop - header.offsetHeight - 100 &&
+						child.offsetTop + child.offsetHeight - hf.scrollTop > 100)
+					Element.show("floatingTitle");
+				else
+					Element.hide("floatingTitle");
+
+				break;
+			}
+		}
+	} catch (e) {
+		exception_error("updateFloatingTitle", e);
+	}
+}
